@@ -1,7 +1,7 @@
 #
 # coding=utf-8
 """versionah - Simple version specification management"""
-# Copyright © 2011, 2012  James Rowe <jnrowe@gmail.com>
+# Copyright © 2011-2014  James Rowe <jnrowe@gmail.com>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -25,7 +25,7 @@ from . import _version
 __version__ = _version.dotted
 __date__ = _version.date
 __author__ = 'James Rowe <jnrowe@gmail.com>'
-__copyright__ = 'Copyright © 2011, 2012, 2013  James Rowe <jnrowe@gmail.com>'
+__copyright__ = 'Copyright (C) 2011-2014  James Rowe <jnrowe@gmail.com>'
 __license__ = 'GNU General Public License Version 3'
 __credits__ = ''
 __history__ = 'See git repository'
@@ -79,26 +79,23 @@ from .i18n import _
 T = Terminal()
 
 #: Base string type, used for compatibility with Python 2 and 3
-STR_TYPE = str if sys.version_info[0] == 3 else basestring
-
-#: Command line help string, for use with :mod:`argparse`
-# Pull the first paragraph from the docstring
-USAGE = '\n'.join(__doc__[:__doc__.find('\n\n', 100)].splitlines()[2:])
-# Replace script name with argparse's substitution var
-USAGE = USAGE.replace('versionah', '%(prog)s')
+STR_TYPE = basestring if sys.version_info[0] == 2 else str
 
 #: Command line interface object
-APP = aaargh.App(description=USAGE,
+APP = aaargh.App(description=_('A tool to manage project version files'),
                  epilog=_('Please report bugs to jnrowe@gmail.com'))
 
 #: Regular expression to match a valid package name
-VALID_PACKAGE = '[A-Za-z][A-Za-z0-9]+(?:[_-][A-Za-z0-9]+)*'
+VALID_PACKAGE = '[A-Za-z][A-Za-z0-9]+(?:[_\.-][A-Za-z0-9]+)*'
 #: Regular expression to match a valid package version
 VALID_VERSION = r'\d+\.\d+(?:\.\d+){,2}'
 #: Regular expression to match a package date.  ISO-8601, and %d-%b-%Y
 #: formatting for shtool compatibility
 VALID_DATE = r'(?:\d{4}-\d{2}-\d{2}|\d{2}-(?:%s)-\d{4})' % '|'.join(MONTHS)
 
+#: Supported version components
+VERSION_COMPS = ('major', 'minor', 'micro', 'patch')
+
 
 class ValidatingAction(argparse.Action):
 
@@ -118,7 +115,6 @@ def success(text):
     """Format a success message with colour, if possible.
 
     :rtype: `str`
-
     """
     return T.bright_green(text)
 
@@ -127,7 +123,6 @@ def fail(text):
     """Format a failure message with colour, if possible.
 
     :rtype: `str`
-
     """
     return T.bright_red(text)
 
@@ -136,7 +131,6 @@ def warn(text):
     """Format a warning message with colour, if possible.
 
     :rtype: `str`
-
     """
     return T.bright_yellow(text)
 
@@ -152,7 +146,6 @@ def filter_regexp(string, pattern, repl, count=0, flags=0):
 
     :rtype: `str`
     :return: Text with substitutions applied
-
     """
     if sys.version_info[:2] >= (2, 7):
         return re.sub(pattern, repl, string, count, flags)
@@ -195,7 +188,6 @@ class Version(object):
         :param components: Version components
         :param str name: Package name
         :param datetime.date date: Date associated with version
-
         """
         if isinstance(components, STR_TYPE):
             components = split_version(components)
@@ -218,7 +210,6 @@ class Version(object):
 
         :rtype: `str`
         :return: String representation of object
-
         """
         return '%s(%r, %r, %r)' % (self.__class__.__name__, self.components,
                                    self.name, self.date)
@@ -230,7 +221,6 @@ class Version(object):
 
         :rtype: `str`
         :return: Default strings representation of object
-
         """
         return '%s v%s' % (self.name, self.as_dotted())
 
@@ -245,7 +235,6 @@ class Version(object):
         :rtype: `tuple`
         :return: Full version component tuple for object
         :raise NotImplementedError: Incomparable other
-
         """
         if isinstance(other, Version):
             return other.components_full
@@ -264,7 +253,6 @@ class Version(object):
         considered equal to 0.1.0.0.
 
         :rtype: `bool`
-
         """
         return self.components_full == self.__prepare_cmp_object(other)
     __ne__ = lambda self, other: not self == (other)
@@ -276,7 +264,6 @@ class Version(object):
 
         :rtype: `bool`
         :return: True if ``self`` is strictly less-than ``other``
-
         """
         return self.components < self.__prepare_cmp_object(other)
 
@@ -287,7 +274,6 @@ class Version(object):
 
         :rtype: `bool`
         :return: True if ``self`` is strictly greater-than ``other``
-
         """
         return self.components_full > self.__prepare_cmp_object(other)
 
@@ -298,7 +284,6 @@ class Version(object):
 
         :rtype: `bool`
         :return: True if ``self`` is less-than or equal to ``other``
-
         """
         return self < other or self == other
 
@@ -309,7 +294,6 @@ class Version(object):
 
         :rtype: `bool`
         :return: True if ``self`` is greater-than or equal to ``other``
-
         """
         return self > other or self == other
 
@@ -318,7 +302,6 @@ class Version(object):
 
         :rtype: `str`
         :return: Object-unique hash value
-
         """
         return hash(repr(self))
 
@@ -327,7 +310,6 @@ class Version(object):
 
         :type components: `tuple` of `int`
         :param components: Version components
-
         """
         if isinstance(components, STR_TYPE):
             components = split_version(components)
@@ -342,7 +324,6 @@ class Version(object):
         """Generate full length component tuple for version.
 
         :rtype: `tuple` of `int`
-
         """
         return self.major, self.minor, self.micro, self.patch
 
@@ -351,19 +332,16 @@ class Version(object):
         """Generate component tuple to initial resolution.
 
         :rtype: `tuple` of `int`
-
         """
-
         return self.components_full[:self._resolution]
 
     def bump(self, bump_type):
         """Bump a version string.
 
         :param str bump_type: Component to bump
-
         """
         if bump_type == 'micro' and self._resolution < 3 \
-            or bump_type == 'patch' and self._resolution < 4:
+                or bump_type == 'patch' and self._resolution < 4:
             raise ValueError('Invalid bump_type %r for version %r'
                              % (bump_type, self.components))
         if bump_type == 'major':
@@ -397,12 +375,19 @@ class Version(object):
         """Bump patch version component."""
         self.bump('patch')
 
+    def as_dict(self):
+        """Generate a dictionary of version components.
+
+        :rtype: `dict`
+        :return: Version as dictionary
+        """
+        return dict(zip(VERSION_COMPS, self.components))
+
     def as_dotted(self):
         """Generate a dotted version string.
 
         :rtype: `str`
         :return: Standard dotted version string
-
         """
         return '.'.join(str(s) for s in self.components)
 
@@ -411,7 +396,6 @@ class Version(object):
 
         :rtype: `str`
         :return: Version as hex string
-
         """
         return '0x' + ''.join('%02x' % n for n in self.components)
 
@@ -420,7 +404,6 @@ class Version(object):
 
         :rtype: `str`
         :return: Version as libtool string
-
         """
         return '%i:%i' % (self.major * 10 + self.minor, 20 + self.micro)
 
@@ -429,7 +412,6 @@ class Version(object):
 
         :rtype: `str`
         :return: Version's release date as ISO-8601 date stamp
-
         """
         return self.date.isoformat()
 
@@ -438,7 +420,6 @@ class Version(object):
 
         :rtype: `int`
         :return: Version components as tuple
-
         """
         return self.components
 
@@ -447,7 +428,6 @@ class Version(object):
 
         :rtype: `str`
         :return: Version's string in web UA-style
-
         """
         return '%s/%s' % (self.name, self.as_dotted())
 
@@ -457,7 +437,6 @@ class Version(object):
 
         :rtype: `list` of `str`
         :return: Method names for representation types
-
         """
         return [s[3:] for s in dir(Version) if s.startswith('as_')]
 
@@ -467,7 +446,6 @@ class Version(object):
         :param str display_format: Format to display version string in
         :rtype: `str`
         :return: Formatted version string
-
         """
         return getattr(self, 'as_%s' % display_format)()
 
@@ -480,9 +458,9 @@ class Version(object):
         :return: New `Version` object representing file
         :raise exceptions.OSError: When ``filename`` doesn't exist
         :raise exceptions.ValueError: Unparsable version data
-
         """
-        data = open(filename).read().strip()
+        with open(filename) as f:
+            data = f.read().strip()
         match = re.search(r'This is (%s),? [vV]ersion (%s) \((%s)\)'
                           % (VALID_PACKAGE, VALID_VERSION, VALID_DATE),
                           data)
@@ -506,7 +484,6 @@ class Version(object):
         :return: `True` on write success
 
         .. _shtool: http://www.gnu.org/software/shtool/shtool.html
-
         """
         data = vars(self)
         data.update({
@@ -519,7 +496,7 @@ class Version(object):
         if shtool:
             # %d-%b-%Y, if %b wasn't locale dependent
             shtool_date = "%s-%s-%s" % (self.date.day,
-                                        MONTHS[self.date.month-1],
+                                        MONTHS[self.date.month - 1],
                                         self.date.year)
             data['magic'] = 'This is %s, Version %s (%s)' % (self.name,
                                                              self.as_dotted(),
@@ -529,13 +506,13 @@ class Version(object):
                                                             self.as_dotted(),
                                                             self.as_date())
 
-        data.update(dict(zip(['major', 'minor', 'micro', 'patch'],
-                             self.components)))
+        data.update(dict(zip(VERSION_COMPS, self.components)))
         data.update(dict([(k[3:], getattr(self, k)())
                           for k in dir(self) if k.startswith('as_')]))
 
         template = self.env.get_template('%s.jinja' % file_type)
-        open(filename, 'w').write(template.render(data))
+        with open(filename, 'w') as f:
+            f.write(template.render(data))
 
 
 def split_version(version):
@@ -545,7 +522,6 @@ def split_version(version):
     :rtype: `tuple` of `int`
     :return: Components of version string
     :raise exceptions.ValueError: Invalid version string
-
     """
     if not re.match('%s$' % VALID_VERSION, version):
         raise ValueError('Invalid version string %r' % version)
@@ -574,15 +550,16 @@ OPTIONS.add_argument('-d', '--display', default='dotted',
 OPTIONS.add_argument('filename', help=_('version file to operate on'))
 
 
-@APP.cmd(help=_('bump version in given file'), parents=[OPTIONS, ])
+@APP.cmd(name='bump', help=_('bump version in given file'),
+         parents=[OPTIONS, ])
 @APP.cmd_arg('-t', '--type', choices=Version.filetypes, dest='file_type',
              metavar='text',
              help=_('define the file type used for version file'))
 @APP.cmd_arg('--shtool', action='store_true',
              help=_('write shtool compatible output'))
-@APP.cmd_arg('bump', choices=('major', 'minor', 'micro', 'patch'),
-             help=_('bump type'))
-def bump(display_format, filename, file_type, shtool, bump):
+@APP.cmd_arg('bump', default=None, nargs='?',
+             choices=('major', 'minor', 'micro', 'patch'), help=_('bump type'))
+def bump_version(display_format, filename, file_type, shtool, bump):
     """Bump version in existing file.
 
     :param str display_format: Format to display output in
@@ -592,7 +569,6 @@ def bump(display_format, filename, file_type, shtool, bump):
     :param str bump: Component to bump
 
     .. _shtool: http://www.gnu.org/software/shtool/shtool.html
-
     """
     if not file_type:
         file_type = guess_type(filename)
@@ -607,13 +583,16 @@ def bump(display_format, filename, file_type, shtool, bump):
         print(fail(_('File not found')))
         return errno.ENOENT
 
+    if not bump:
+        bump = VERSION_COMPS[len(version.components) - 1]
+
     version.bump(bump)
     version.write(filename, file_type, shtool)
 
     print(success(version.display(display_format)))
 
 
-@APP.cmd(help=_('set version in given file'), parents=[OPTIONS, ])
+@APP.cmd(name='set', help=_('set version in given file'), parents=[OPTIONS, ])
 @APP.cmd_arg('-t', '--type', choices=Version.filetypes, dest='file_type',
              metavar='text',
              help=_('define the file type used for version file'))
@@ -621,17 +600,16 @@ def bump(display_format, filename, file_type, shtool, bump):
              metavar=os.path.basename(os.getenv('PWD')),
              action=ValidatingAction,
              help=_('package name for version(default from $PWD)'))
-@APP.cmd_arg('set', metavar='0.1.0', action=ValidatingAction,
+@APP.cmd_arg('version_str', metavar='version', action=ValidatingAction,
              help='set to a specific version')
-def set(display_format, filename, file_type, name, set):
+def set_version(display_format, filename, file_type, name, version_str):
     """Set version in new or existing file.
 
     :param str display_format: Format to display output in
     :param str filename: File to operate on
     :param str file_type: File type to produce
     :param str name: Project name used in output
-    :param str set: Initial version string
-
+    :param str version_str: Initial version string
     """
     if not file_type:
         file_type = guess_type(filename)
@@ -647,7 +625,7 @@ def set(display_format, filename, file_type, name, set):
     if name:
         version.name = name
 
-    version.set(set)
+    version.set(version_str)
     version.write(filename, file_type)
 
     print(success(version.display(display_format)))
@@ -659,20 +637,21 @@ def display(display_format, filename):
 
     :param str display_format: Format to display output in
     :param str filename: File to operate on
-
     """
     try:
         version = Version.read(filename)
     except IOError as error:
         print(fail(error.args[1]))
         return errno.EEXIST
+    except ValueError as error:
+        print(fail(error.args[0]))
+        return errno.EIO
 
     print(success(version.display(display_format)))
 
 
 def main():
     """Main script entry point."""
-
     APP.arg('--version', action='version',
             version='%%(prog)s %s' % _version.dotted)
-    APP.run()
+    return APP.run()
